package com.landray.kmss.sys.attachment.thirdSfs.rest;

import com.landray.kmss.framework.plugin.core.config.IExtension;
import com.landray.kmss.framework.plugin.core.config.IExtensionPoint;
import com.landray.kmss.util.SpringBeanUtil;
import com.landray.kmss.framework.service.plugin.Plugin;
import java.util.Iterator;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import com.landray.kmss.common.rest.controller.BaseController;

public class SysAttThirdController<T extends SysAttExtendThirdRestService> extends BaseController
{
    protected static List<StandardThirdRestService> standardThirdRestServices;
    protected static List<ExtendThirdRestService> extendThirdRestServices;
    
    static {
        SysAttThirdController.standardThirdRestServices = new ArrayList<StandardThirdRestService>();
        SysAttThirdController.extendThirdRestServices = new ArrayList<ExtendThirdRestService>();
    }
    
    protected SysAttStandardThirdRestService getSysAttStandardThirdRestService(final HttpServletRequest request) {
        String url = request.getRequestURI();
        url = url.replace(request.getContextPath(), "");
        String urlKey = url.split("/")[3];
        if ("yozoAtt".equals(urlKey)) {
            urlKey = "yozo";
        }
        final List<StandardThirdRestService> standardThirdRestServices = this.getStandardThirdRestServices();
        for (final StandardThirdRestService restService : standardThirdRestServices) {
            if (urlKey.equals(restService.getRestKey())) {
                return restService.getRestService();
            }
        }
        return null;
    }
    
    protected T getSysAttExtendThirdRestService(final HttpServletRequest request) {
        String url = request.getRequestURI();
        url = url.replace(request.getContextPath(), "");
        String urlKey = url.split("/")[3];
        if ("yozoAtt".equals(urlKey)) {
            urlKey = "yozo";
        }
        final List<ExtendThirdRestService> extendThirdRestServices = this.getExtendThirdRestServices();
        for (final ExtendThirdRestService restService : extendThirdRestServices) {
            if (urlKey.equals(restService.getRestKey())) {
                return (T)restService.getRestService();
            }
        }
        return null;
    }
    
    protected List<StandardThirdRestService> getStandardThirdRestServices() {
        if (SysAttThirdController.standardThirdRestServices.isEmpty()) {
            synchronized (SysAttThirdController.standardThirdRestServices) {
                if (SysAttThirdController.standardThirdRestServices.isEmpty()) {
                    final IExtensionPoint point = Plugin.getExtensionPoint("com.landray.kmss.sys.attachment.third.sfs.service");
                    final IExtension[] extensions = point.getExtensions();
                    IExtension[] array;
                    for (int length = (array = extensions).length, i = 0; i < length; ++i) {
                        final IExtension extension = array[i];
                        if ("restStandard".equals(extension.getAttribute("name"))) {
                            final String key = Plugin.getParamValueString(extension, "restKey");
                            final String name = Plugin.getParamValueString(extension, "name");
                            final String serviceName = Plugin.getParamValueString(extension, "service");
                            final SysAttStandardThirdRestService restService = (SysAttStandardThirdRestService)SpringBeanUtil.getBean(serviceName);
                            SysAttThirdController.standardThirdRestServices.add(new StandardThirdRestService(key, name, restService));
                        }
                    }
                }
                monitorexit(SysAttThirdController.standardThirdRestServices);
            }
        }
        return SysAttThirdController.standardThirdRestServices;
    }
    
    protected List<ExtendThirdRestService> getExtendThirdRestServices() {
        if (SysAttThirdController.extendThirdRestServices.isEmpty()) {
            synchronized (SysAttThirdController.extendThirdRestServices) {
                if (SysAttThirdController.extendThirdRestServices.isEmpty()) {
                    final IExtensionPoint point = Plugin.getExtensionPoint("com.landray.kmss.sys.attachment.third.sfs.service");
                    final IExtension[] extensions = point.getExtensions();
                    IExtension[] array;
                    for (int length = (array = extensions).length, i = 0; i < length; ++i) {
                        final IExtension extension = array[i];
                        if ("restExtend".equals(extension.getAttribute("name"))) {
                            final String key = Plugin.getParamValueString(extension, "restKey");
                            final String name = Plugin.getParamValueString(extension, "name");
                            final String serviceName = Plugin.getParamValueString(extension, "service");
                            final SysAttExtendThirdRestService restService = (SysAttExtendThirdRestService)SpringBeanUtil.getBean(serviceName);
                            SysAttThirdController.extendThirdRestServices.add(new ExtendThirdRestService(key, name, restService));
                        }
                    }
                }
                monitorexit(SysAttThirdController.extendThirdRestServices);
            }
        }
        return SysAttThirdController.extendThirdRestServices;
    }
    
    static class StandardThirdRestService
    {
        private String restKey;
        private String name;
        private SysAttStandardThirdRestService restService;
        
        public StandardThirdRestService(final String restKey, final String name, final SysAttStandardThirdRestService restService) {
            this.restKey = restKey;
            this.name = name;
            this.restService = restService;
        }
        
        public String getRestKey() {
            return this.restKey;
        }
        
        public String getName() {
            return this.name;
        }
        
        public SysAttStandardThirdRestService getRestService() {
            return this.restService;
        }
    }
    
    static class ExtendThirdRestService
    {
        private String restKey;
        private String name;
        private SysAttExtendThirdRestService restService;
        
        public ExtendThirdRestService(final String restKey, final String name, final SysAttExtendThirdRestService restService) {
            this.restKey = restKey;
            this.name = name;
            this.restService = restService;
        }
        
        public String getRestKey() {
            return this.restKey;
        }
        
        public String getName() {
            return this.name;
        }
        
        public SysAttExtendThirdRestService getRestService() {
            return this.restService;
        }
    }
}
